    console.log("Loading mraid_internal.js");

    (function () {
        // Root bridge object handler
        var
            mraidinternal = window.mraid_internal = {},
            bridge = window.mraidbridge,
            // Use a callback id to prevent multiple callbacks overlapping if
            // multiple callers want the same value.
            nextCallbackId = 0,
            callbacks = {},
            contactsData = {},
            handleNativeEvent = function (json) {
                if (!json) {
                    return;
                }
                var callbackId = json.callbackId;
                console.log("mraidinternal handleNativeEvent listener - callbackId=" + callbackId);
                if (callbacks[callbackId]) {
                    callbacks[callbackId](json.data);
                } else {
                    console.log("mraidinternal - missing native handler event for callbackId: " + callbackId); 
                }
            }
        ;

        //////////////////////////////////////////////////////////////////////////////////////////////////
        // MRAID get contacts

        // Register listeners to be triggered when native call completes.
        bridge.addEventListener('getContactsCompleted', function (json) {
            console.log("mraidbridge.getContactsCompleted called");

            var
                callbackId = json.callbackId,
                data = contactsData[callbackId] || []
            ;
            data = data.concat(json.data.contacts);
            if (json.data.hasMore) {
                contactsData[callbackId] = data;
            } else {
                delete contactsData[callbackId];
								
                json.data = {
                    "contacts": data,
                    "totalContacts": json.data.totalContacts
                };
                handleNativeEvent(json);
            }
        });
        bridge.addEventListener('getContactPhotosCompleted', function (json) {
            console.log("mraidbridge.getContactPhotosCompleted called");
            handleNativeEvent(json);
        });

        /*
         * Asynchronously fetches device contacts that have at least 1 phone number and returns in JSON format:
         *
         * {
         *     "contacts": [
         *         {
         *             id: "15",
         *             name: "John Smith",
         *             hasPhoto: true,
         *             numbers: [
         *                 { type: "mobile", number: "1-250-345-6789" },
         *                 { type: "work", number: "1.250.222.4444" }
         *             ]
         *         },
         *         {
         *             id: "135",
         *             name: "Jane Doe",
         *             hasPhoto: false,
         *             numbers: [
         *                 { type: "home", number: "12505436767" }
         *             ]
         *         }
         *         ...
         *     ],
         *     "totalContacts": 53
         * }
         *
         * @param offset      - Within the total set of all contacts found, offset specifies the position
         *                      of the first contact fetched.
         * @param maxContacts - Maximum number contacts to fetch.
         * @param callback    - Contacts will be returned in callback. totalContacts specifies total 
         *                      number of contacts found.
         *
         */
        mraidinternal.getContacts = function (offset, maxContacts, callback) {
            // Save the callback.
            var callbackId = nextCallbackId++;
            callbacks[callbackId] = callback;
            bridge.executeNativeCallWithScheme('mraidinternal', 'getcontacts', 'offset', offset, 'maxContacts', maxContacts, 'callbackId', callbackId);
        };

        /*
         * Asynchronously fetches contact photos from the device and returns in JSON format:
         *
         * {
         *     "id": "<contactId>",
         *     "photo": "<base64 encoded photo>",
         *     "hasMore" true/false
         * }
         *
         * @param contactIds - array of contactIds to fetch photo for.
         * @param callback - Photos will be returned one at a time. The callback will be called
         *                   for each photo and passed the data as described above. 
         *                   Check the data for hasMore=false to know when you have all photos.
         *
         */
        mraidinternal.getContactPhotos = function (contactIds, callback) {
            // Save the callback.
            var callbackId = nextCallbackId++;
            callbacks[callbackId] = callback;
            bridge.executeNativeCallWithScheme(
                'mraidinternal',
                'getcontactphotos',
                'contactIds', JSON.stringify(contactIds),
                'callbackId', callbackId
            );
        };

        //////////////////////////////////////////////////////////////////////////////////////////////////
        // MRAID Local Cache
        
        // Register listener to be triggered when native call completes.
        bridge.addEventListener('cacheGetCompletedEvent', function(json) {
            console.log("mraidbridge.cacheGetCompletedEvent - json=" + stringify(json));
            handleNativeEvent(json);
        });

        /*
         * Get the value indexed at key from the cache.
         *
         * @param key - index of value in the cache.
         * @param callback - callback returns a single JSON string param.
         */
        mraidinternal.cacheGet = function (key, callback) {
            // Save the callback.
            var callbackId = nextCallbackId++;
            callbacks[callbackId] = callback;
            console.log("mraidinternal.cacheGet - key=" + key + ", callbackId=" + callbackId);
            bridge.executeNativeCallWithScheme('mraidinternal', 'cacheGet', 'key', key, 'callbackId', callbackId);
        };

        /*
         * Set the value indexed at key in the cache.
         *
         * @param key - index of value in the cache.
         * @param value - value to be stored in cache. value must be a JSON string.
         */
        mraidinternal.cacheSet = function (key, value) {
            console.log("mraidinternal.cacheSet - key=" + key + ", value=" + value);
            bridge.executeNativeCallWithScheme('mraidinternal', 'cacheSet', 'key', key, 'value', value);
        };

        /*
         * Remove the value indexed at key from the cache.
         *
         * @param key - index of value in the cache.
         */
        mraidinternal.cacheRemove = function (key) {
            console.log("mraidinternal.cacheRemove - key=" + key);
            bridge.executeNativeCallWithScheme('mraidinternal', 'cacheRemove', 'key', key);
        };

    }());
